"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const body_parser_1 = __importDefault(require("body-parser"));
const express_1 = __importDefault(require("express"));
const fs_1 = __importDefault(require("fs"));
// import * as fs from 'fs';
// import AsapRequest from './asap_request';
const config_1 = __importDefault(require("./config"));
const handlers_1 = __importDefault(require("./handlers"));
const logger_1 = __importDefault(require("./logger"));
const authorization_1 = require("./middleware/authorization");
const errorHandler = __importStar(require("./middleware/error_handler"));
const middleware_utils_1 = require("./middleware/middleware_utils");
const token_generator_1 = __importDefault(require("./token_generator"));
const asap_1 = require("./util/asap");
const context = __importStar(require("./util/context"));
const issToBaseUrlMapping = new Map();
for (const issuer of config_1.default.SystemAsapJwtAcceptedHookIss.values()) {
    issToBaseUrlMapping.set(issuer, config_1.default.SystemAsapBaseUrlMappings);
}
const systemJwtClaims = {
    asapJwtAcceptedAud: config_1.default.SystemAsapJwtAcceptedAud,
    asapJwtAcceptedHookIss: config_1.default.SystemAsapJwtAcceptedHookIss
};
const jitsiJwtClaims = {
    asapJwtAcceptedAud: config_1.default.SystemAsapJwtAcceptedAud,
    asapJwtAcceptedHookIss: config_1.default.SystemAsapJwtAcceptedHookIss
};
const asapFetcher = new asap_1.ASAPPubKeyFetcher(issToBaseUrlMapping, new RegExp(config_1.default.KidPrefixPattern), config_1.default.AsapPubKeyTTL);
const generatorAuthorization = new authorization_1.GeneratorAuthorization({
    asapFetcher,
    protectedApi: config_1.default.ProtectedApi,
    systemJwtClaims,
    jitsiJwtClaims
});
const jwtSigningKey = fs_1.default.readFileSync(config_1.default.AsapSigningKeyFile);
const tokenGenerator = new token_generator_1.default({
    signingKey: jwtSigningKey,
    asapJwtIss: config_1.default.AsapJwtIss,
    asapJwtAud: config_1.default.AsapJwtAud,
    asapJwtKid: config_1.default.AsapJwtKid,
    asapJwtSub: config_1.default.AsapJwtSub
});
const handlers = new handlers_1.default({
    tokenGenerator,
    salt: config_1.default.KeyGenSalt
});
const app = express_1.default();
app.get('/health', (req, res) => {
    res.send('OK');
});
app.use(body_parser_1.default.json());
app.use(express_1.default.json());
app.use('/', context.injectContext);
logger_1.default.info(`Starting up jitsi-token-generator service with config: ${JSON.stringify(config_1.default)}`);
// Use system authorization for the rest of endpoints
const noSystemAuthPaths = [
    '/jitsi-token-generator/blah'
];
app.use(middleware_utils_1.unless(generatorAuthorization.systemAuthMiddleware, ...noSystemAuthPaths));
// This is placed last in the middleware chain and is our default error handler.
app.use(errorHandler.errorHandlerMiddleware);
app.get('/generate/client', async (req, res, next) => {
    try {
        await handlers.generateClientToken(req, res);
    }
    catch (err) {
        next(err);
    }
});
app.get('/generate/component', async (req, res, next) => {
    try {
        await handlers.generateComponentToken(req, res);
    }
    catch (err) {
        next(err);
    }
});
app.get('/generate/server', async (req, res, next) => {
    try {
        await handlers.generateServerToken(req, res);
    }
    catch (err) {
        next(err);
    }
});
app.post('/generate/client', async (req, res, next) => {
    try {
        await handlers.generateClientToken(req, res);
    }
    catch (err) {
        next(err);
    }
});
app.post('/generate/component', async (req, res, next) => {
    try {
        await handlers.generateComponentToken(req, res);
    }
    catch (err) {
        next(err);
    }
});
app.post('/generate/server', async (req, res, next) => {
    try {
        await handlers.generateServerToken(req, res);
    }
    catch (err) {
        next(err);
    }
});
app.listen(config_1.default.HTTPServerPort, () => {
    logger_1.default.info(`...listening on :${config_1.default.HTTPServerPort}`);
});
