"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dotenv = __importStar(require("dotenv"));
const envalid_1 = __importDefault(require("envalid"));
const crypto = require('crypto');
const result = dotenv.config();
if (result.error) {
    const err = result.error;
    switch (err.code) {
        case 'ENOENT':
            // skip if only error is missing file, this isn't fatal
            console.debug('Missing .env file, not loading environment file disk');
            break;
        default:
            throw result.error;
    }
}
/**
 * Converts a string to Uint8Array.
 * @param str
 */
function asciiToUint8Array([...str]) {
    return new Uint8Array(str.map(char => char.charCodeAt(0)));
}
const env = envalid_1.default.cleanEnv(process.env, {
    ASAP_JWT_AUD: envalid_1.default.str({ default: 'jitsi' }),
    ASAP_JWT_ISS: envalid_1.default.str({ default: 'jitsi-token-generator' }),
    ASAP_JWT_KID: envalid_1.default.str(),
    ASAP_JWT_SUB: envalid_1.default.str({ default: undefined }),
    ASAP_PUB_KEY_TTL: envalid_1.default.num({ default: 3600 }),
    ASAP_SIGNING_KEY_FILE: envalid_1.default.str(),
    HOSTNAME: envalid_1.default.str({ default: '' }),
    KEY_GEN_SALT: envalid_1.default.str({ default: crypto.randomBytes(16).toString('hex') }),
    KEY_PREFIX_PATTERN: envalid_1.default.str({ default: '^(.*)/(.*)$' }),
    LOG_LEVEL: envalid_1.default.str({ default: 'info' }),
    PORT: envalid_1.default.num({ default: 8017 }),
    PROTECTED_API: envalid_1.default.bool({ default: true }),
    REQUEST_RETRY_COUNT: envalid_1.default.num({ default: 2 }),
    REQUEST_TIMEOUT_MS: envalid_1.default.num({ default: 8000 }),
    SYSTEM_ASAP_BASE_URL_MAPPINGS: envalid_1.default.json({ example: '[{"kid": "kidPattern", baseUrl": "https://jaas-public-keys.jitsi.net/server/dev"}]'
    }),
    SYSTEM_ASAP_JWT_ACCEPTED_HOOK_ISS: envalid_1.default.str(),
    SYSTEM_ASAP_JWT_AUD: envalid_1.default.str()
});
exports.default = {
    AsapJwtAud: env.ASAP_JWT_AUD,
    AsapJwtIss: env.ASAP_JWT_ISS,
    AsapJwtKid: env.ASAP_JWT_KID,
    AsapJwtSub: env.ASAP_JWT_SUB,
    AsapPubKeyTTL: env.ASAP_PUB_KEY_TTL,
    AsapSigningKeyFile: env.ASAP_SIGNING_KEY_FILE,
    HTTPServerPort: env.PORT,
    Hostname: env.HOSTNAME,
    KeyGenSalt: asciiToUint8Array(env.KEY_GEN_SALT),
    KidPrefixPattern: env.KEY_PREFIX_PATTERN,
    LogLevel: env.LOG_LEVEL,
    ProtectedApi: env.PROTECTED_API,
    RequestRetryCount: env.REQUEST_RETRY_COUNT,
    RequestTimeoutMs: env.REQUEST_TIMEOUT_MS,
    SystemAsapBaseUrlMappings: env.SYSTEM_ASAP_BASE_URL_MAPPINGS,
    SystemAsapJwtAcceptedAud: env.SYSTEM_ASAP_JWT_AUD,
    SystemAsapJwtAcceptedHookIss: env.SYSTEM_ASAP_JWT_ACCEPTED_HOOK_ISS.split(',')
};
