"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// import got, { CancelableRequest } from 'got';
const jsonwebtoken_1 = require("jsonwebtoken");
const node_cache_1 = __importDefault(require("node-cache"));
/**
 */
class TokenGenerator {
    /**
     * Constructor
     * @param options
     */
    constructor(options) {
        this.cacheTTL = 60 * 45;
        this.signingKey = options.signingKey;
        this.asapJwtIss = options.asapJwtIss;
        this.asapJwtAud = options.asapJwtAud;
        this.asapJwtKid = options.asapJwtKid;
        this.asapJwtSub = options.asapJwtSub;
        this.signingKeys = {};
        if (options.signingKeys !== undefined) {
            this.signingKeys = options.signingKeys;
        }
        if (options.cacheTTL !== undefined) {
            this.cacheTTL = options.cacheTTL;
        }
        this.asapCache = new node_cache_1.default({ stdTTL: this.cacheTTL }); // TTL of 45 minutes
        this.defaultSignOptions = {
            algorithm: 'RS256',
            expiresIn: '1 hour',
            notBefore: '-5m',
            audience: this.asapJwtAud,
            issuer: this.asapJwtIss,
            keyid: this.asapJwtKid
        };
        if (this.asapJwtSub) {
            this.defaultSignOptions.subject = this.asapJwtSub;
        }
        this.serverToken = this.serverToken.bind(this);
    }
    /**
     *
     * @param ctx Context
     * @param payload JWT claims
     * @param options ASAP signing options
     * @returns string for server auth
     */
    serverToken(ctx, payload = {}, options = {}) {
        // const cacheKey = 'asap_server';
        // const cachedAuth: string = this.asapCache.get(cacheKey);
        // if (cachedAuth) {
        //     return cachedAuth;
        // }
        ctx.logger.debug('serverToken generation');
        const auth = this.signJWT(ctx, payload, options);
        // this.asapCache.set(cacheKey, auth);
        return auth;
    }
    /**
     *
     * @param ctx Context
     * @param payload JWT claims
     * @param options ASAP signing options
     * @returns string for client auth
     */
    clientToken(ctx, payload, options = {}) {
        return this.signJWT(ctx, payload, options);
    }
    /**
     * signJWT creates and returns a new JWT
     * @param ctx Context
     * @param options SignOptions
     */
    signJWT(ctx, claims, options) {
        ctx.logger.debug('new JWT generation');
        const signOptions = {
            ...this.defaultSignOptions,
            ...options
        };
        // pull the signing key by kid, if not found use the default key
        const signingKey = this.signingKeys[signOptions.keyid] ? this.signingKeys[signOptions.keyid] : this.signingKey;
        const auth = jsonwebtoken_1.sign(claims, signingKey, signOptions);
        return auth;
    }
}
exports.default = TokenGenerator;
