"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.accessLogger = exports.injectContext = exports.generateNewContext = exports.Context = void 0;
const jitsiLogger = __importStar(require("@jitsi/logger"));
const shortid_1 = __importDefault(require("shortid"));
/**
 * The request context, used for injecting request id into logs
 */
class Context {
    /**
     * Constructor
     * @param reqLogger logger instance for this request
     * @param start start date of the request
     * @param requestId the request identifier
     */
    constructor(reqLogger, start, requestId) {
        this.logger = reqLogger;
        this.start = start;
        this.requestId = requestId;
    }
}
exports.Context = Context;
/**
 * Generate new request context
 * @param contextId id of the context
 */
function generateNewContext(contextId) {
    const start = Date.now();
    let resultedContextId = contextId;
    if (!resultedContextId) {
        resultedContextId = shortid_1.default.generate();
    }
    const ctxLogger = jitsiLogger.getUntrackedLogger(resultedContextId, undefined, {});
    return new Context(ctxLogger, start, resultedContextId);
}
exports.generateNewContext = generateNewContext;
/**
 * Adds context to the express request object. The context
 * includes the start date, a request id and a handler specific logger.
 * This middleware should be registered before any middleware that make use of
 * context or anything it contains.
 * @param req request
 * @param res response
 * @param next next
 */
function injectContext(req, res, next) {
    const requestIdHeader = req.header('X-Request-Id');
    const start = Date.now();
    const reqId = requestIdHeader ? requestIdHeader : shortid_1.default.generate();
    const reqLogger = jitsiLogger.getUntrackedLogger(reqId, undefined, {});
    req.context = new Context(reqLogger, start, reqId);
    res.header('X-Request-Id', reqId);
    next();
}
exports.injectContext = injectContext;
/**
 * Logs summary data for each http api call. It makes use of the context logger.
 * @param req request
 * @param res response
 * @param next next
 */
function accessLogger(req, res, next) {
    let logged = false;
    const accessLog = function () {
        if (!logged) {
            logged = true;
            const requestInfo = JSON.stringify({
                m: req.method,
                u: req.originalUrl,
                s: res.statusCode,
                d: Math.abs(Date.now() - req.context.start)
            });
            req.context.logger.info(`${requestInfo}`);
        }
    };
    res.on('finish', accessLog);
    res.on('close', accessLog);
    next();
}
exports.accessLogger = accessLogger;
