"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateKey = void 0;
const crypto = require('crypto').webcrypto;
/**
 * Generates key that can be used for e2ee, using some salt param.
 * @param value - The value to use to generate key.
 * @param salt - The salt.
 */
async function generateKey(value, salt) {
    const keyMaterial = await crypto.subtle.importKey('raw', Buffer.from(value, 'utf8'), 'PBKDF2', false, ['deriveKey']);
    const dk = await crypto.subtle.deriveKey({
        name: 'PBKDF2',
        salt,
        iterations: 100000,
        hash: 'SHA-256'
    }, keyMaterial, {
        name: 'AES-GCM',
        length: 128
    }, true, ['encrypt', 'decrypt']);
    const toHexString = (bytes) => Array.from(new Uint8Array(bytes), (byte) => 
    // eslint-disable-next-line no-bitwise
    `0${(byte & 0xff).toString(16)}`.slice(-2)).join('');
    return toHexString(await crypto.subtle.exportKey('raw', dk));
}
exports.generateKey = generateKey;
